@@ -25,8 +25,10 @@
 
 #include "changerecorder.h"
 #include "controladaptor.h"
+#include "dbusconnectionpool.h"
 #include "itemfetchjob.h"
 #include "monitor_p.h"
+#include "servermanager_p.h"
 #include "session.h"
 #include "session_p.h"
 #include "statusadaptor.h"
@@ -47,6 +49,8 @@
 #include <signal.h>
 #include <stdlib.h>
 
+//#define EXPERIMENTAL_INPROCESS_AGENTS 1
+
 using namespace Akonadi;
 
 static AgentBase *sAgentBase = 0;
@@ -117,16 +121,24 @@ void AgentBase::ObserverV2::itemLinked( const Akonadi::Item& item, const Akonadi
 {
   Q_UNUSED( item );
   Q_UNUSED( collection );
-  if ( sAgentBase != 0 )
+  if ( sAgentBase != 0 ) {
+    // not implementation, let's disconnect the signal to enable optimizations in Monitor
+    QObject::disconnect( sAgentBase->changeRecorder(), SIGNAL( itemLinked( const Akonadi::Item&, const Akonadi::Collection& ) ),
+                         sAgentBase->d_ptr, SLOT( itemLinked( const Akonadi::Item&, const Akonadi::Collection& ) ) );
     sAgentBase->d_ptr->changeProcessed();
+  }
 }
 
 void AgentBase::ObserverV2::itemUnlinked( const Akonadi::Item& item, const Akonadi::Collection& collection )
 {
   Q_UNUSED( item );
   Q_UNUSED( collection );
-  if ( sAgentBase != 0 )
+  if ( sAgentBase != 0 ) {
+    // not implementation, let's disconnect the signal to enable optimizations in Monitor
+    QObject::disconnect( sAgentBase->changeRecorder(), SIGNAL( itemUnlinked( const Akonadi::Item&, const Akonadi::Collection& ) ),
+                         sAgentBase->d_ptr, SLOT( itemUnlinked( const Akonadi::Item&, const Akonadi::Collection& ) ) );
     sAgentBase->d_ptr->changeProcessed();
+  }
 }
 
 void AgentBase::ObserverV2::collectionMoved( const Akonadi::Collection &collection, const Akonadi::Collection &source, const Akonadi::Collection &dest )
@@ -148,6 +160,7 @@ void AgentBase::ObserverV2::collectionChanged( const Akonadi::Collection &collec
 
 AgentBasePrivate::AgentBasePrivate( AgentBase *parent )
   : q_ptr( parent ),
+    mDBusConnection( QString() ),
     mStatusCode( AgentBase::Idle ),
     mProgress( 0 ),
     mNeedsNetwork( false ),
@@ -155,11 +168,15 @@ AgentBasePrivate::AgentBasePrivate( AgentBase *parent )
     mSettings( 0 ),
     mObserver( 0 )
 {
+#ifdef _WIN32_WCE
+  QThread::currentThread()->setPriority(QThread::LowPriority);
+#endif
+  Internal::setClientType( Internal::Agent );
 }
 
 AgentBasePrivate::~AgentBasePrivate()
 {
-  mMonitor->setConfig( 0 );
+  mChangeRecorder->setConfig( 0 );
   delete mSettings;
 }
 
@@ -172,23 +189,32 @@ void AgentBasePrivate::init()
    */
   SessionPrivate::createDefaultSession( mId.toLatin1() );
 
-  mTracer = new org::freedesktop::Akonadi::Tracer( QLatin1String( "org.freedesktop.Akonadi" ), QLatin1String( "/tracing" ),
-                                           QDBusConnection::sessionBus(), q );
+  if ( QThread::currentThread() != QCoreApplication::instance()->thread() ) {
+    mDBusConnection = QDBusConnection::connectToBus( QDBusConnection::SessionBus, q->identifier() );
+    Q_ASSERT( mDBusConnection.isConnected() );
+  }
 
-  new ControlAdaptor( q );
-  new StatusAdaptor( q );
-  if ( !QDBusConnection::sessionBus().registerObject( QLatin1String( "/" ), q, QDBusConnection::ExportAdaptors ) )
-    q->error( QString::fromLatin1( "Unable to register object at dbus: %1" ).arg( QDBusConnection::sessionBus().lastError().message() ) );
+  mTracer = new org::freedesktop::Akonadi::Tracer( QLatin1String( "org.freedesktop.Akonadi" ),
+                                                   QLatin1String( "/tracing" ),
+                                                   DBusConnectionPool::threadConnection(), q );
+
+  new Akonadi__ControlAdaptor( q );
+  new Akonadi__StatusAdaptor( q );
+  if ( !DBusConnectionPool::threadConnection().registerObject( QLatin1String( "/" ), q, QDBusConnection::ExportAdaptors ) )
+    q->error( QString::fromLatin1( "Unable to register object at dbus: %1" ).arg( DBusConnectionPool::threadConnection().lastError().message() ) );
 
   mSettings = new QSettings( QString::fromLatin1( "%1/agent_config_%2" ).arg( XdgBaseDirs::saveDir( "config", QLatin1String( "akonadi" ) ), mId ), QSettings::IniFormat );
 
-  mMonitor = new ChangeRecorder( q );
-  mMonitor->ignoreSession( Session::defaultSession() );
-  mMonitor->itemFetchScope().setCacheOnly( true );
-  mMonitor->setConfig( mSettings );
+  mChangeRecorder = new ChangeRecorder( q );
+  mChangeRecorder->ignoreSession( Session::defaultSession() );
+  mChangeRecorder->itemFetchScope().setCacheOnly( true );
+  mChangeRecorder->setConfig( mSettings );
 
   mOnline = mSettings->value( QLatin1String( "Agent/Online" ), true ).toBool();
 
+  // reinitialize the status message now that online state is available
+  mStatusMessage = defaultReadyMessage();
+
   mName = mSettings->value( QLatin1String( "Agent/Name" ) ).toString();
   if ( mName.isEmpty() ) {
     mName = mSettings->value( QLatin1String( "Resource/Name" ) ).toString();
@@ -198,28 +224,32 @@ void AgentBasePrivate::init()
     }
   }
 
-  connect( mMonitor, SIGNAL( itemAdded( const Akonadi::Item&, const Akonadi::Collection& ) ),
+  connect( mChangeRecorder, SIGNAL( itemAdded( const Akonadi::Item&, const Akonadi::Collection& ) ),
            SLOT( itemAdded( const Akonadi::Item&, const Akonadi::Collection& ) ) );
-  connect( mMonitor, SIGNAL( itemChanged( const Akonadi::Item&, const QSet<QByteArray>& ) ),
+  connect( mChangeRecorder, SIGNAL( itemChanged( const Akonadi::Item&, const QSet<QByteArray>& ) ),
            SLOT( itemChanged( const Akonadi::Item&, const QSet<QByteArray>& ) ) );
-  connect( mMonitor, SIGNAL( itemMoved( const Akonadi::Item&, const Akonadi::Collection&, const Akonadi::Collection& ) ),
+  connect( mChangeRecorder, SIGNAL( itemMoved( const Akonadi::Item&, const Akonadi::Collection&, const Akonadi::Collection& ) ),
            SLOT( itemMoved( const Akonadi::Item&, const Akonadi::Collection&, const Akonadi::Collection& ) ) );
-  connect( mMonitor, SIGNAL( itemRemoved( const Akonadi::Item& ) ),
+  connect( mChangeRecorder, SIGNAL( itemRemoved( const Akonadi::Item& ) ),
            SLOT( itemRemoved( const Akonadi::Item& ) ) );
-  connect( mMonitor, SIGNAL( collectionAdded( const Akonadi::Collection&, const Akonadi::Collection& ) ),
+  connect( mChangeRecorder, SIGNAL( collectionAdded( const Akonadi::Collection&, const Akonadi::Collection& ) ),
            SLOT( collectionAdded( const Akonadi::Collection&, const Akonadi::Collection& ) ) );
-  connect( mMonitor, SIGNAL( itemLinked( const Akonadi::Item&, const Akonadi::Collection& ) ),
+  connect( mChangeRecorder, SIGNAL( itemLinked( const Akonadi::Item&, const Akonadi::Collection& ) ),
            SLOT( itemLinked( const Akonadi::Item&, const Akonadi::Collection& ) ) );
-  connect( mMonitor, SIGNAL( itemUnlinked( const Akonadi::Item&, const Akonadi::Collection& ) ),
+  connect( mChangeRecorder, SIGNAL( itemUnlinked( const Akonadi::Item&, const Akonadi::Collection& ) ),
            SLOT( itemUnlinked( const Akonadi::Item&, const Akonadi::Collection& ) ) );
-  connect( mMonitor, SIGNAL( collectionChanged( const Akonadi::Collection& ) ),
+  connect( mChangeRecorder, SIGNAL( collectionChanged( const Akonadi::Collection& ) ),
            SLOT( collectionChanged( const Akonadi::Collection& ) ) );
-  connect( mMonitor, SIGNAL( collectionChanged( const Akonadi::Collection&, const QSet<QByteArray>& ) ),
+  connect( mChangeRecorder, SIGNAL( collectionChanged( const Akonadi::Collection&, const QSet<QByteArray>& ) ),
            SLOT( collectionChanged( const Akonadi::Collection&, const QSet<QByteArray>& ) ) );
-  connect( mMonitor, SIGNAL( collectionMoved( const Akonadi::Collection&, const Akonadi::Collection&, const Akonadi::Collection& ) ),
+  connect( mChangeRecorder, SIGNAL( collectionMoved( const Akonadi::Collection&, const Akonadi::Collection&, const Akonadi::Collection& ) ),
            SLOT( collectionMoved( const Akonadi::Collection&, const Akonadi::Collection&, const Akonadi::Collection& ) ) );
-  connect( mMonitor, SIGNAL( collectionRemoved( const Akonadi::Collection& ) ),
+  connect( mChangeRecorder, SIGNAL( collectionRemoved( const Akonadi::Collection& ) ),
            SLOT( collectionRemoved( const Akonadi::Collection& ) ) );
+  connect( mChangeRecorder, SIGNAL( collectionSubscribed( const Akonadi::Collection&, const Akonadi::Collection& ) ),
+           SLOT( collectionSubscribed( const Akonadi::Collection&, const Akonadi::Collection& ) ) );
+  connect( mChangeRecorder, SIGNAL( collectionUnsubscribed( const Akonadi::Collection& ) ),
+           SLOT( collectionUnsubscribed( const Akonadi::Collection& ) ) );
 
   connect( q, SIGNAL( status( int, const QString& ) ), q, SLOT( slotStatus( int, const QString& ) ) );
   connect( q, SIGNAL( percent( int ) ), q, SLOT( slotPercent( int ) ) );
@@ -229,11 +259,14 @@ void AgentBasePrivate::init()
   // Use reference counting to allow agents to finish internal jobs when the
   // agent is stopped.
   KGlobal::ref();
-  KGlobal::setAllowQuit( true );
+  if ( QThread::currentThread() == QCoreApplication::instance()->thread() )
+    KGlobal::setAllowQuit( true );
 
+#ifndef Q_OS_WINCE
   // disable session management
   if ( KApplication::kApplication() )
     KApplication::kApplication()->disableSessionManagement();
+#endif
 
   QTimer::singleShot( 0, q, SLOT( delayedInit() ) );
 }
@@ -241,8 +274,8 @@ void AgentBasePrivate::init()
 void AgentBasePrivate::delayedInit()
 {
   Q_Q( AgentBase );
-  if ( !QDBusConnection::sessionBus().registerService( QLatin1String( "org.freedesktop.Akonadi.Agent." ) + mId ) )
-    kFatal() << "Unable to register service at dbus:" << QDBusConnection::sessionBus().lastError().message();
+  if ( !DBusConnectionPool::threadConnection().registerService( QLatin1String( "org.freedesktop.Akonadi.Agent." ) + mId ) )
+    kFatal() << "Unable to register service at dbus:" << DBusConnectionPool::threadConnection().lastError().message();
   q->setOnline( mOnline );
 }
 
@@ -265,8 +298,11 @@ void AgentBasePrivate::itemMoved( const Akonadi::Item &item, const Akonadi::Coll
     // inter-resource moves, requires we know which resources the source and destination are in though
     if ( !source.resource().isEmpty() && !dest.resource().isEmpty() ) {
       if ( source.resource() != dest.resource() ) {
-        if ( source.resource() == q_ptr->identifier() ) // moved away from us
-          mObserver->itemRemoved( item );
+        if ( source.resource() == q_ptr->identifier() ) { // moved away from us
+          Akonadi::Item i( item );
+          i.setParentCollection( source );
+          mObserver->itemRemoved( i );
+        }
         else if ( dest.resource() == q_ptr->identifier() ) // moved to us
           mObserver->itemAdded( item, dest );
         else if ( observer2 )
@@ -370,10 +406,23 @@ void AgentBasePrivate::collectionRemoved( const Akonadi::Collection &collection
     mObserver->collectionRemoved( collection );
 }
 
+void AgentBasePrivate::collectionSubscribed( const Akonadi::Collection &collection, const Akonadi::Collection &parent )
+{
+  Q_UNUSED( collection );
+  Q_UNUSED( parent );
+  changeProcessed();
+}
+
+void AgentBasePrivate::collectionUnsubscribed( const Akonadi::Collection &collection )
+{
+  Q_UNUSED( collection );
+  changeProcessed();
+}
+
 void AgentBasePrivate::changeProcessed()
 {
-  mMonitor->changeProcessed();
-  QTimer::singleShot( 0, mMonitor, SLOT( replayNext() ) );
+  mChangeRecorder->changeProcessed();
+  QTimer::singleShot( 0, mChangeRecorder, SLOT( replayNext() ) );
 }
 
 void AgentBasePrivate::slotStatus( int status, const QString &message )
@@ -424,10 +473,9 @@ void AgentBasePrivate::slotError( const QString& message )
 void AgentBasePrivate::slotNetworkStatusChange( Solid::Networking::Status stat )
 {
   Q_Q( AgentBase );
-  q->setOnline( stat == Solid::Networking::Connected );
+  q->setOnline( stat == Solid::Networking::Unknown || stat == Solid::Networking::Connected );
 }
 
-
 AgentBase::AgentBase( const QString & id )
   : d_ptr( new AgentBasePrivate( this ) )
 {
@@ -574,7 +622,7 @@ void AgentBase::configure( qlonglong windowId )
 
 WId AgentBase::winIdForDialogs() const
 {
-  const bool registered = QDBusConnection::sessionBus().interface()->isServiceRegistered( QLatin1String( "org.freedesktop.akonaditray" ) );
+  const bool registered = DBusConnectionPool::threadConnection().interface()->isServiceRegistered( QLatin1String( "org.freedesktop.akonaditray" ) );
   if ( !registered )
     return 0;
 
@@ -596,7 +644,7 @@ void AgentBase::quit()
   aboutToQuit();
 
   if ( d->mSettings ) {
-    d->mMonitor->setConfig( 0 );
+    d->mChangeRecorder->setConfig( 0 );
     d->mSettings->sync();
   }
 
@@ -612,7 +660,7 @@ void AgentBase::cleanup()
   Q_D( AgentBase );
   // prevent the monitor from picking up deletion signals for our own data if we are a resource
   // and thus avoid that we kill our own data as last act before our own death
-  d->mMonitor->blockSignals( true );
+  d->mChangeRecorder->blockSignals( true );
 
   aboutToQuit();
 
@@ -621,7 +669,7 @@ void AgentBase::cleanup()
   /*
    * First destroy the settings object...
    */
-  d->mMonitor->setConfig( 0 );
+  d->mChangeRecorder->setConfig( 0 );
   delete d->mSettings;
   d->mSettings = 0;
 
@@ -630,10 +678,15 @@ void AgentBase::cleanup()
    */
   QFile::remove( fileName );
 
+  /*
+   * ... and remove the changes file from hd.
+   */
+  QFile::remove( fileName + QLatin1String( "_changes.dat" ) );
+
   /*
    * ... and also remove the agent configuration file if there is one.
    */
-  QString configFile = KStandardDirs::locateLocal( "config", KGlobal::config()->name() );
+  QString configFile = KStandardDirs::locateLocal( "config", config()->name() );
   QFile::remove( configFile );
 
   KGlobal::deref();
@@ -641,6 +694,7 @@ void AgentBase::cleanup()
 
 void AgentBase::registerObserver( Observer *observer )
 {
+  // TODO in theory we should re-connect change recorder signals here that we disconnected previously
   d_ptr->mObserver = observer;
 }
 
@@ -686,7 +740,15 @@ void AgentBase::changeProcessed()
 
 ChangeRecorder * AgentBase::changeRecorder() const
 {
-  return d_ptr->mMonitor;
+  return d_ptr->mChangeRecorder;
+}
+
+KSharedConfigPtr AgentBase::config()
+{
+  if ( QCoreApplication::instance()->thread() == QThread::currentThread() )
+    return KGlobal::config();
+  else
+    return componentData().config();
 }
 
 void AgentBase::abort()
@@ -699,5 +761,20 @@ void AgentBase::reconfigure()
   emit reloadConfiguration();
 }
 
+extern QThreadStorage<KComponentData*> s_agentComponentDatas;
+
+KComponentData AgentBase::componentData()
+{
+  if ( QThread::currentThread() == QCoreApplication::instance()->thread() ) {
+    if ( s_agentComponentDatas.hasLocalData() )
+      return *(s_agentComponentDatas.localData());
+    else
+      return KGlobal::mainComponent();
+  }
+
+  Q_ASSERT( s_agentComponentDatas.hasLocalData() );
+  return *(s_agentComponentDatas.localData());
+}
+
 #include "agentbase.moc"
 #include "agentbase_p.moc"
