@@ -25,6 +25,7 @@
 #include "tracerinterface.h"
 
 #include <klocale.h>
+
 #include <solid/networking.h>
 
 class QSettings;
@@ -74,6 +75,9 @@ class AgentBasePrivate : public QObject
     QString mId;
     QString mName;
 
+    /// Use sessionBus() to access the connection.
+    QDBusConnection mDBusConnection;
+
     int mStatusCode;
     QString mStatusMessage;
 
@@ -104,6 +108,8 @@ class AgentBasePrivate : public QObject
     void collectionChanged( const Akonadi::Collection &collection, const QSet<QByteArray> &partIdentifiers );
     void collectionMoved( const Akonadi::Collection &collection, const Akonadi::Collection &source, const Akonadi::Collection &destination );
     void collectionRemoved( const Akonadi::Collection &collection );
+    void collectionSubscribed( const Akonadi::Collection &collection, const Akonadi::Collection &parent );
+    void collectionUnsubscribed( const Akonadi::Collection &collection );
 };
 
 }
