@@ -46,7 +46,7 @@ class EntityTreeModelPrivate;
  * Akonadi models and views provide a high level way to interact with the akonadi server.
  * Most applications will use these classes.
  *
- * Models provide an interface for viewing, deleting and moving Items and Collections.
+ * Models provide an interface for viewing, updating, deleting and moving Items and Collections.
  * Additionally, the models are updated automatically if another application changes the
  * data or inserts of deletes items etc.
  *
@@ -66,19 +66,7 @@ class EntityTreeModelPrivate;
  * and the EntityTreeModel.
  *
  * If you want to retrieve a Collection for a particular Collection::Id  and you do not yet
- * have a valid QModelIndex, use match:
- *
- * @code
- * QModelIndexList list = model->match( QModelIndex(), CollectionIdRole, id );
- * if ( list.isEmpty() )
- *   return; // A Collection with that Id is not in the model.
- * Q_ASSERT( list.size() == 1 ); // Otherwise there must be only one instance of it.
- * Collection collection = list.at( 0 ).data( EntityTreeModel::CollectionRole ).value<Collection>();
- * @endcode
- *
- * Not that a single Item may appear multiple times in a model, so the list size may not be 1
- * if it is not empty in that case, so the Q_ASSERT should not be used.
- * @see virtual-collections
+ * have a valid QModelIndex, use modelIndexForCollection.
  *
  * <h3>Using EntityTreeModel in your application</h3>
  *
@@ -180,6 +168,15 @@ class EntityTreeModelPrivate;
  *
  * This feature is used in KAddressBook.
  *
+ * If items are to be fetched by the model, it is necessary to specify which parts of the items
+ * are to be fetched, using the ItemFetchScope class. By default, only the basic metadata is
+ * fetched. To fetch all item data, including all attributes:
+ *
+ * @code
+ * changeRecorder->itemFetchScope().fetchFullPayload();
+ * changeRecorder->itemFetchScope().fetchAllAttributes();
+ * @endcode
+ *
  * <h2>Using EntityTreeModel with Proxy models</h2>
  *
  * An Akonadi::SelectionProxyModel can be used to simplify managing selection in one view through multiple proxy models to a representation in another view.
@@ -304,6 +301,16 @@ class EntityTreeModelPrivate;
  * itemList->setHeaderGroup( EntityTreeModel::ItemListHeaders );
  * @endcode
  *
+ * <h3>Progress reporting</h3>
+ *
+ * The EntityTreeModel uses asynchronous Akonadi::Job instances to fill and update itself.
+ * For example, a job is run to fetch the contents of collections (that is, list the items in it).
+ * Additionally, individual Akonadi::Items can be fetched in different parts at different times.
+ *
+ * To indicate that such a job is underway, the EntityTreeModel makes the FetchState available. The
+ * FetchState returned from a QModelIndex representing a Akonadi::Collection will be FetchingState if a
+ * listing of the items in that collection is underway, otherwise the state is IdleState.
+ *
  * @author Stephen Kelly <steveire@gmail.com>
  * @since 4.4
  */
@@ -339,11 +346,23 @@ class AKONADI_EXPORT EntityTreeModel : public QAbstractItemModel
       CollectionDerefRole,                    ///< @internal Used to decrease the reference count on a Collection
       PendingCutRole,                         ///< @internal Used to indicate items which are to be cut
       EntityUrlRole,                          ///< The akonadi:/ Url of the entity as a string. Item urls will contain the mimetype.
+      UnreadCountRole,                        ///< Returns the number of unread items in a collection. @since 4.5
+      FetchStateRole,                         ///< Returns the FetchState of a particular item. @since 4.5
+      CollectionSyncProgressRole,             ///< Returns the progress of synchronization in percent for a perticular collection. @since 4.6
       UserRole = Qt::UserRole + 500,          ///< First role for user extensions.
       TerminalUserRole = 2000,                ///< Last role for user extensions. Don't use a role beyond this or headerData will break.
       EndRole = 65535
     };
 
+    /**
+     * Describes the state of fetch jobs related to particular entities.
+     * @since 4.5
+     */
+    enum FetchState {
+      IdleState,                              ///< There is no fetch in progress.
+      FetchingState                           ///< There is a fetch in progress.
+      // TODO: Change states for reporting of fetching payload parts of items.
+    };
 
     /**
      * Describes what header information the model shall return.
@@ -394,6 +413,27 @@ class AKONADI_EXPORT EntityTreeModel : public QAbstractItemModel
      */
     bool systemEntitiesShown() const;
 
+
+    /**
+     * Returns whether unsubscribed entities will be included in the listing.
+     *
+     * @since 4.5
+     */
+    bool includeUnsubscribed() const;
+
+    /**
+     * Sets whether unsubscribed entities will be included in the listing.
+     * By default it's true
+     *
+     * Note that it is possible to change the monitor's fetchscope directly,
+     *  bypassing this method, which will lead to inconsistencies. Use
+     *  this method for turning on/off listing of subscribed folders.
+     *
+     * @since 4.5
+     */
+    void setIncludeUnsubscribed( bool show );
+
+
     /**
      * Sets the item population @p strategy of the model.
      */
@@ -436,7 +476,8 @@ class AKONADI_EXPORT EntityTreeModel : public QAbstractItemModel
     enum CollectionFetchStrategy {
       FetchNoCollections,               ///< Fetches nothing. This creates an empty model.
       FetchFirstLevelChildCollections,  ///< Fetches first level collections in the root collection.
-      FetchCollectionsRecursive         ///< Fetches collections in the root collection recursively. This is the default.
+      FetchCollectionsRecursive,        ///< Fetches collections in the root collection recursively. This is the default.
+      InvisibleCollectionFetch          ///< Fetches collections, but does not put them in the model. This can be used to create a list of items in all collections. The ParentCollectionRole can still be used to retrieve the parent collection of an Item. @since 4.5
     };
 
     /**
@@ -476,6 +517,48 @@ class AKONADI_EXPORT EntityTreeModel : public QAbstractItemModel
      */
     virtual QModelIndexList match( const QModelIndex& start, int role, const QVariant& value, int hits = 1, Qt::MatchFlags flags = Qt::MatchFlags( Qt::MatchStartsWith | Qt::MatchWrap ) ) const;
 
+    /**
+     * Returns a QModelIndex in @p model which points to @p collection.
+     * This method can be used through proxy models if @p model is a proxy model.
+     *
+     * @code
+     * EntityTreeModel *model = getEntityTreeModel();
+     * QSortFilterProxyModel *proxy1 = new QSortFilterProxyModel;
+     * proxy1->setSourceModel(model);
+     * QSortFilterProxyModel *proxy2 = new QSortFilterProxyModel;
+     * proxy2->setSourceModel(proxy1);
+     *
+     * ...
+     *
+     * QModelIndex idx = EntityTreeModel::modelIndexForCollection(proxy2, Collection(colId));
+     * if (!idx.isValid())
+     *   // Collection with id colId is not in the proxy2.
+     *   // Maybe it is filtered out if proxy 2 is only showing items? Make sure you use the correct proxy.
+     *   return;
+     *
+     * Collection collection = idx.data( EntityTreeModel::CollectionRole ).value<Collection>();
+     * // collection has the id colId, and all other attributes already fetched by the model such as name, remoteId, Akonadi::Attributes etc.
+     *
+     * @endcode
+     *
+     * This can be useful for example if an id is stored in a config file and needs to be used in the application.
+     *
+     * Note however, that to restore view state such as scrolling, selection and expansion of items in trees, the ETMViewStateSaver can be used for convenience.
+     *
+     * @see modelIndexesForItem
+     * @since 4.5
+     */
+    static QModelIndex modelIndexForCollection( const QAbstractItemModel *model, const Collection &collection );
+
+    /**
+     * Returns a QModelIndex in @p model which points to @p item.
+     * This method can be used through proxy models if @p model is a proxy model.
+     *
+     * @see modelIndexForCollection
+     * @since 4.5
+     */
+    static QModelIndexList modelIndexesForItem( const QAbstractItemModel *model, const Item &item );
+
   protected:
     /**
      * Clears and resets the model. Always call this instead of the reset method in the superclass.
@@ -531,25 +614,31 @@ private:
 
     Q_PRIVATE_SLOT( d_func(), void rootCollectionFetched(Akonadi::Collection::List) )
     Q_PRIVATE_SLOT( d_func(), void startFirstListJob() )
-    // Q_PRIVATE_SLOT( d_func(), void slotModelReset() )
+    Q_PRIVATE_SLOT( d_func(), void serverStarted() )
 
     // TODO: Can I merge these into one jobResult slot?
     Q_PRIVATE_SLOT( d_func(), void fetchJobDone( KJob *job ) )
     Q_PRIVATE_SLOT( d_func(), void pasteJobDone( KJob *job ) )
     Q_PRIVATE_SLOT( d_func(), void updateJobDone( KJob *job ) )
+    Q_PRIVATE_SLOT( d_func(), void firstFetchJobDone( KJob *job ) )
 
     Q_PRIVATE_SLOT( d_func(), void itemsFetched( Akonadi::Item::List ) )
     Q_PRIVATE_SLOT( d_func(), void collectionsFetched( Akonadi::Collection::List ) )
+    Q_PRIVATE_SLOT( d_func(), void firstCollectionsFetched( Akonadi::Collection::List ) )
+    Q_PRIVATE_SLOT( d_func(), void collectionListFetched( Akonadi::Collection::List ) )
     Q_PRIVATE_SLOT( d_func(), void topLevelCollectionsFetched( Akonadi::Collection::List ) )
     Q_PRIVATE_SLOT( d_func(), void ancestorsFetched( Akonadi::Collection::List ) )
 
     Q_PRIVATE_SLOT( d_func(), void monitoredMimeTypeChanged( const QString&, bool ) )
+    Q_PRIVATE_SLOT( d_func(), void monitoredCollectionsChanged( const Akonadi::Collection&, bool ) )
 
     Q_PRIVATE_SLOT( d_func(), void monitoredCollectionAdded( const Akonadi::Collection&, const Akonadi::Collection& ) )
     Q_PRIVATE_SLOT( d_func(), void monitoredCollectionRemoved( const Akonadi::Collection& ) )
     Q_PRIVATE_SLOT( d_func(), void monitoredCollectionChanged( const Akonadi::Collection& ) )
     Q_PRIVATE_SLOT( d_func(), void monitoredCollectionMoved( const Akonadi::Collection&, const Akonadi::Collection&,
                                                              const Akonadi::Collection&) )
+    Q_PRIVATE_SLOT( d_func(), void collectionSubscribed( const Akonadi::Collection&, const Akonadi::Collection& ) )
+    Q_PRIVATE_SLOT( d_func(), void monitoredCollectionUnsubscribed( const Akonadi::Collection& ) )
 
     Q_PRIVATE_SLOT( d_func(), void monitoredItemAdded( const Akonadi::Item&, const Akonadi::Collection& ) )
     Q_PRIVATE_SLOT( d_func(), void monitoredItemRemoved( const Akonadi::Item& ) )
@@ -559,6 +648,9 @@ private:
 
     Q_PRIVATE_SLOT( d_func(), void monitoredItemLinked( const Akonadi::Item&, const Akonadi::Collection& ) )
     Q_PRIVATE_SLOT( d_func(), void monitoredItemUnlinked( const Akonadi::Item&, const Akonadi::Collection& ) )
+    Q_PRIVATE_SLOT( d_func(), void changeFetchState( const Akonadi::Collection & ) )
+
+    Q_PRIVATE_SLOT( d_func(), void agentInstanceAdvancedStatusChanged( const QString&, const QVariantMap& ) )
     //@endcond
 };
 
