@@ -22,12 +22,15 @@
 #include <kconfiggroup.h>
 #include <kdebug.h>
 #include <kglobal.h>
-#include <klibloader.h>
 #include <klocale.h>
 #include <kstandarddirs.h>
+#include <KPluginLoader>
 
 #include <QtCore/QDebug>
-#include <QtCore/QPluginLoader>
+
+#ifdef _WIN32_WCE
+ #include <QMessageBox>
+#endif
 
 using namespace Akonadi;
 
@@ -35,9 +38,10 @@ PluginMetaData::PluginMetaData()
 {
 }
 
-PluginMetaData::PluginMetaData( const QString & lib, const QString & name, const QString & comment )
+PluginMetaData::PluginMetaData( const QString & lib, const QString & name, const QString & comment, const QString & cname )
   : library( lib ), nameLabel( name ),
-    descriptionLabel( comment ), loaded( false )
+    descriptionLabel( comment ),
+    className(cname), loaded( false )
 {
 }
 
@@ -63,47 +67,70 @@ PluginLoader* PluginLoader::self()
   return mSelf;
 }
 
-QStringList PluginLoader::types() const
+QStringList PluginLoader::names() const
 {
   return mPluginInfos.keys();
 }
 
-QObject* PluginLoader::createForName( const QString &type )
+QObject* PluginLoader::createForName( const QString & name )
 {
-  if ( !mPluginInfos.contains( type ) ) {
-    kWarning( 5300 ) << "plugin type \"" << type << "\" is unknown to the plugin loader." << endl;
+  if ( !mPluginInfos.contains( name ) ) {
+    kWarning( 5300 ) << "plugin name \"" << name << "\" is unknown to the plugin loader." << endl;
     return 0;
   }
 
-  PluginMetaData &info = mPluginInfos[ type ];
+  PluginMetaData &info = mPluginInfos[ name ];
+
+  //First try to load it staticly
+  foreach (QObject *plugin, QPluginLoader::staticInstances()) {
+    if(QLatin1String(plugin->metaObject()->className()) == info.className) {
+      info.loaded = true;
+      return plugin;
+      break;
+    }
+  }
+
   if ( !info.loaded ) {
-    const QString path = KLibLoader::findLibrary( info.library );
-    if ( path.isEmpty() ) {
-      kWarning( 5300 ) << "unable to find library for plugin type \"" << type << "\"." << endl;
+    KPluginLoader* loader = new KPluginLoader( info.library );
+    if ( loader->fileName().isEmpty() ) {
+      kWarning( 5300 ) << loader->errorString();
+      delete loader;
       return 0;
     }
 
-    mPluginLoaders.insert( type, new QPluginLoader( path ) );
+    mPluginLoaders.insert( name, loader );
     info.loaded = true;
   }
 
-  QPluginLoader *loader = mPluginLoaders[ type ];
+  QPluginLoader *loader = mPluginLoaders.value( name );
+  Q_ASSERT(loader);
 
   QObject *object = loader->instance();
   if ( !object ) {
-    kWarning( 5300 ) << "unable to load plugin for plugin type \"" << type << "\"." << endl;
+#ifdef _WIN32_WCE
+	//Maybe filter out the default plugins, they should be found but...
+    //if ( !name.endsWith( QLatin1String( "@default" ) ) ) {
+      QString errMessage;
+      errMessage.append(QLatin1String("plugin \""));
+      errMessage.append(info.className);
+      errMessage.append(QLatin1String("\" is not buildin static, please specify this information in the bugreport."));
+      QMessageBox::critical(NULL,QLatin1String("Error"), errMessage);
+    //}
+#endif
+    kWarning( 5300 ) << "unable to load plugin for plugin name \"" << name << "\"." << endl;
+    kWarning( 5300 ) << "Error was:\"" << loader->errorString() << "\"." << endl;
     return 0;
   }
 
   return object;
 }
 
-PluginMetaData PluginLoader::infoForName( const QString & type ) const
+PluginMetaData PluginLoader::infoForName( const QString & name ) const
 {
-  if ( !mPluginInfos.contains( type ) )
+  if ( !mPluginInfos.contains( name ) )
     return PluginMetaData();
 
-  return mPluginInfos.value( type );
+  return mPluginInfos.value( name );
 }
 
 void PluginLoader::scan()
@@ -123,6 +150,15 @@ void PluginLoader::scan()
         continue;
       }
 
+      // read Class entry as a list so that types like QPair<A,B> are
+      // properly escaped and don't end up being split into QPair<A
+      // and B>.
+      const QStringList classes = group.readXdgListEntry( "X-Akonadi-Class" );
+      if ( classes.isEmpty() ) {
+        kWarning( 5300 ) << "missing or empty [Plugin]X-Akonadi-Class value in \"" << entry << "\" - skipping" << endl;
+        continue;
+      }
+
       const QString library = group.readEntry( "X-KDE-Library" );
       if ( library.isEmpty() ) {
         kWarning( 5300 ) << "missing or empty [Plugin]X-KDE-Library value in \"" << entry << "\" - skipping" << endl;
@@ -134,22 +170,27 @@ void PluginLoader::scan()
       QString name = group2.readEntry( "Name" );
       if ( name.isEmpty() ) {
         kWarning( 5300 ) << "missing or empty [Misc]Name value in \"" << entry << "\" - inserting default name" << endl;
-        name = i18n("Unnamed plugin");
+        name = i18n( "Unnamed plugin" );
       }
 
       QString comment = group2.readEntry( "Comment" );
       if ( comment.isEmpty() ) {
         kWarning( 5300 ) << "missing or empty [Misc]Comment value in \"" << entry << "\" - inserting default name" << endl;
-        comment = i18n("No description available");
+        comment = i18n( "No description available" );
+      }
+      
+      QString cname      = group.readEntry( "X-KDE-ClassName" );
+      if ( cname.isEmpty() ) {
+        kWarning( 5300 ) << "missing or empty X-KDE-ClassName value in \"" << entry << "\"" << endl;
       }
 
-      if ( type.contains( QLatin1Char( ',' ) ) ) {
-        const QStringList subTypes = type.split( QLatin1Char( ',' ), QString::SkipEmptyParts );
-        for ( int j = 0; j < subTypes.count(); ++j )
-          mPluginInfos.insert( subTypes.at( j ), PluginMetaData( library, name, comment ) );
-      } else {
-        mPluginInfos.insert( type, PluginMetaData( library, name, comment ) );
-      }
+      const QStringList mimeTypes = type.split( QLatin1Char( ',' ), QString::SkipEmptyParts );
+
+      kDebug( 5300 ) << "registering Desktop file" << entry << "for" << mimeTypes << '@' << classes;
+      Q_FOREACH( const QString & mimeType, mimeTypes )
+        Q_FOREACH( const QString & classType, classes )
+          mPluginInfos.insert( mimeType + QLatin1Char('@') + classType, PluginMetaData( library, name, comment, cname ) );
+
     } else {
       kWarning( 5300 ) << "Desktop file \"" << entry << "\" doesn't seem to describe a plugin " << "(misses Misc and/or Plugin group)" << endl;
     }
