@@ -43,7 +43,8 @@ class TransactionSequencePrivate;
  * @code
  *
  * // Delete a couple of items inside a transaction
- * Akonadi::TransactionSequence *transaction = new TransactionSequence;
+ * Akonadi::TransactionSequence *transaction = new Akonadi::TransactionSequence;
+ * connect( transaction, SIGNAL( result( KJob* ) ), SLOT( transactionFinished( KJob* ) ) );
  *
  * const Akonadi::Item::List items = ...
  *
@@ -51,7 +52,15 @@ class TransactionSequencePrivate;
  *   new Akonadi::ItemDeleteJob( item, transaction );
  * }
  *
- * transaction->exec();
+ * ...
+ *
+ * MyClass::transactionFinished( KJob *job )
+ * {
+ *   if ( job->error() )
+ *     qDebug() << "Error occurred";
+ *   else
+ *     qDebug() << "Items deleted successfully";
+ * }
  *
  * @endcode
  *
@@ -78,6 +87,35 @@ class AKONADI_EXPORT TransactionSequence : public Job
      */
     void commit();
 
+    /**
+     * Rolls back the current transaction as soon as possible.
+     * You only need to call this method when you want to roll back due to external
+     * reasons (e.g. user cancellation), the transaction is automatically rolled back
+     * if one of its subjobs fails.
+     * @since 4.5
+     */
+    void rollback();
+
+    /**
+     * Sets which job of the sequence might fail without rolling back the
+     * complete transaction.
+     *
+     * @since 4.5
+     */
+    void setIgnoreJobFailure( KJob *job );
+
+    /**
+     * Disable automatic committing.
+     * Use this when you want to add jobs to this sequence after execution
+     * has been started, usually that is outside of the contstructor or the
+     * method that creates this transaction sequence.
+     * @note Calling this method after execution of this job has been started
+     * has no effect.
+     * @param enable @c true to enable autocommitting (default), @c false to disable it
+     * @since 4.5
+     */
+    void setAutomaticCommittingEnabled( bool enable );
+
   protected:
     bool addSubjob( KJob* job );
     void doStart();
