@@ -25,6 +25,7 @@
 #include <gpgme++/context.h>
 #include <gpgme++/eventloopinteractor.h>
 #include <gpgme++/trustitem.h>
+#include <gpgme++/assuanresult.h>
 #include <gpgme++/keylistresult.h>
 #include <gpgme++/keygenerationresult.h>
 #include <gpgme++/importresult.h>
@@ -34,6 +35,10 @@
 #include <gpgme++/encryptionresult.h>
 #include <gpgme++/engineinfo.h>
 #include <gpgme++/editinteractor.h>
+#include <gpgme++/vfsmountresult.h>
+
+#include <gpgme++/interfaces/assuantransaction.h>
+#include <gpgme++/defaultassuantransaction.h>
 
 #include "callbacks.h"
 #include "data_p.h"
@@ -42,6 +47,8 @@
 
 #include <gpgme.h>
 
+#include <boost/scoped_array.hpp>
+
 #include <istream>
 #ifndef NDEBUG
 #include <iostream>
@@ -52,12 +59,49 @@ using std::endl;
 #include <cassert>
 
 namespace GpgME {
+
+  static inline unsigned int xtoi_1( const char * str ) {
+      const unsigned int ch = *str;
+      const unsigned int result =
+          ch <= '9' ? ch - '0' :
+          ch <= 'F' ? ch - 'A' + 10 :
+          /* else */  ch - 'a' + 10 ;
+      return result < 16 ? result : 0 ;
+  }
+  static inline int xtoi_2( const char * str ) {
+      return xtoi_1( str ) * 16U + xtoi_1( str+1 );
+  }
+
+#ifdef HAVE_GPGME_ASSUAN_ENGINE
+  static void percent_unescape( std::string & s, bool plus2space ) {
+      std::string::iterator src = s.begin(), dest = s.begin(), end = s.end();
+      while ( src != end )
+          if ( *src == '%' && end - src > 2 ) {
+              *dest++ = xtoi_2( &*++src );
+              src += 2;
+          } else if ( *src == '+' && plus2space ) {
+              *dest++ = ' ';
+              ++src;
+          } else {
+              *dest++ = *src++;
+          }
+      s.erase( dest, end );
+  }
+#endif
+
   void initializeLibrary() {
       gpgme_check_version( 0 );
   }
 
-  static inline gpgme_error_t makeError( gpg_err_code_t code ) {
-    return gpg_err_make( (gpg_err_source_t)22, code );
+  Error initializeLibrary( int ) {
+#ifndef _WIN32_WCE
+      if ( gpgme_check_version( GPGME_VERSION ) )
+#endif
+          return Error();
+#ifndef _WIN32_WCE
+      else
+          return Error::fromCode( GPG_ERR_USER_1 );
+#endif
   }
 
   static void format_error( gpgme_error_t err, std::string & str ) {
@@ -89,16 +133,73 @@ namespace GpgME {
     return code() == GPG_ERR_CANCELED;
   }
 
+  int Error::toErrno() const {
+//#ifdef HAVE_GPGME_GPG_ERROR_WRAPPERS
+    return gpgme_err_code_to_errno( static_cast<gpgme_err_code_t>( code() ) );
+//#else
+//    return gpg_err_code_to_errno( static_cast<gpg_err_code_t>( code() ) );
+//#endif
+  }
+
+  // static
+  bool Error::hasSystemError() {
+#ifdef HAVE_GPGME_GPG_ERROR_WRAPPERS
+    return gpgme_err_code_from_syserror() == GPG_ERR_MISSING_ERRNO ;
+#else
+    return gpg_err_code_from_syserror() == GPG_ERR_MISSING_ERRNO ;
+#endif
+  }
+
+  // static
+  void Error::setSystemError( gpg_err_code_t err ) {
+    setErrno( gpgme_err_code_to_errno( err ) );
+  }
+
+  // static
+  void Error::setErrno( int err ) {
+#ifdef HAVE_GPGME_GPG_ERROR_WRAPPERS
+    gpgme_err_set_errno( err );
+#else
+    gpg_err_set_errno( err );
+#endif
+  }
+
+  // static
+  Error Error::fromSystemError( unsigned int src ) {
+#ifdef HAVE_GPGME_GPG_ERROR_WRAPPERS
+    return Error( gpgme_err_make( static_cast<gpgme_err_source_t>( src ), gpgme_err_code_from_syserror() ) );
+#else
+    return Error( gpg_err_make( static_cast<gpg_err_source_t>( src ), gpg_err_code_from_syserror() ) );
+#endif
+  }
+
+  // static
+  Error Error::fromErrno( int err, unsigned int src ) {
+//#ifdef HAVE_GPGME_GPG_ERROR_WRAPPERS
+    return Error( gpgme_err_make( static_cast<gpgme_err_source_t>( src ), gpgme_err_code_from_errno( err ) ) );
+//#else
+//    return Error( gpg_err_make( static_cast<gpg_err_source_t>( src ), gpg_err_from_from_errno( err ) ) );
+//#endif
+  }
+
+  // static
+  Error Error::fromCode( unsigned int err, unsigned int src ) {
+//#ifdef HAVE_GPGME_GPG_ERROR_WRAPPERS
+    return Error( gpgme_err_make( static_cast<gpgme_err_source_t>( src ), static_cast<gpgme_err_code_t>( err ) ) );
+//#else
+//    return Error( gpg_err_make( static_cast<gpg_err_source_t>( src ), static_cast<gpgme_err_code_t>( err ) ) );
+//#endif
+  }
+
   std::ostream & operator<<( std::ostream & os, const Error & err ) {
       return os << "GpgME::Error(" << err.encodedError() << " (" << err.asString() << "))";
   }
 
-  Context::Context( gpgme_ctx_t ctx ) {
-    d = new Private( ctx );
+  Context::Context( gpgme_ctx_t ctx ) : d( new Private( ctx ) ) {
   }
 
   Context::~Context() {
-    delete d; d = 0;
+    delete d;
   }
 
   Context * Context::createForProtocol( Protocol proto ) {
@@ -126,6 +227,55 @@ namespace GpgME {
     return new Context( ctx );
   }
 
+  std::auto_ptr<Context> Context::createForEngine( Engine eng, Error * error ) {
+    gpgme_ctx_t ctx = 0;
+    if ( const gpgme_error_t err = gpgme_new( &ctx ) ) {
+        if ( error )
+            *error = Error( err );
+        return std::auto_ptr<Context>();
+    }
+
+    switch ( eng ) {
+    case AssuanEngine:
+#ifdef HAVE_GPGME_ASSUAN_ENGINE
+      if ( const gpgme_error_t err = gpgme_set_protocol( ctx, GPGME_PROTOCOL_ASSUAN ) ) {
+        gpgme_release( ctx );
+        if ( error )
+          *error = Error( err );
+        return std::auto_ptr<Context>();
+      }
+      break;
+#else
+      if ( error )
+        *error = Error::fromCode( GPG_ERR_NOT_SUPPORTED );
+      return std::auto_ptr<Context>();
+#endif
+    case G13Engine:
+#ifdef HAVE_GPGME_G13_VFS
+      if ( const gpgme_error_t err = gpgme_set_protocol( ctx, GPGME_PROTOCOL_G13 ) ) {
+        gpgme_release( ctx );
+        if ( error )
+          *error = Error( err );
+        return std::auto_ptr<Context>();
+      }
+      break;
+#else
+      if ( error )
+        *error = Error::fromCode( GPG_ERR_NOT_SUPPORTED );
+      return std::auto_ptr<Context>();
+#endif
+    default:
+      if ( error )
+        *error = Error::fromCode( GPG_ERR_INV_ARG );
+      return std::auto_ptr<Context>();
+    }
+
+    if ( error )
+        *error = Error();
+
+    return std::auto_ptr<Context>( new Context( ctx ) );
+  }
+
   //
   //
   // Context::Private
@@ -137,6 +287,8 @@ namespace GpgME {
         iocbs( 0 ),
         lastop( None ),
         lasterr( GPG_ERR_NO_ERROR ),
+        lastAssuanInquireData( Data::null ),
+        lastAssuanTransaction(),
         lastEditInteractor(),
         lastCardEditInteractor()
   {
@@ -282,7 +434,7 @@ namespace GpgME {
       const char * const home_dir = engineInfo().homeDirectory();
       return Error( gpgme_ctx_set_engine_info( d->ctx, gpgme_get_protocol( d->ctx ), filename, home_dir ) );
 #else
-      return Error( makeError( GPG_ERR_NOT_IMPLEMENTED ) );
+      return Error::fromCode( GPG_ERR_NOT_IMPLEMENTED );
 #endif
   }
 
@@ -291,7 +443,7 @@ namespace GpgME {
       const char * const filename = engineInfo().fileName();
       return Error( gpgme_ctx_set_engine_info( d->ctx, gpgme_get_protocol( d->ctx ), filename, home_dir ) );
 #else
-      return Error( makeError( GPG_ERR_NOT_IMPLEMENTED ) );
+      return Error::fromCode( GPG_ERR_NOT_IMPLEMENTED );
 #endif
   }
 
@@ -401,12 +553,77 @@ namespace GpgME {
     return ImportResult( d->ctx, Error(d->lasterr) );
   }
 
+  ImportResult Context::importKeys( const std::vector<Key> & kk ) {
+    d->lastop = Private::Import;
+    d->lasterr = make_error( GPG_ERR_NOT_IMPLEMENTED );
+
+    bool shouldHaveResult = false;
+#ifdef HAVE_GPGME_OP_IMPORT_KEYS
+    const boost::scoped_array<gpgme_key_t> keys( new gpgme_key_t[ kk.size() + 1 ] );
+    gpgme_key_t * keys_it = &keys[0];
+    for ( std::vector<Key>::const_iterator it = kk.begin(), end = kk.end() ; it != end ; ++it )
+      if ( it->impl() )
+        *keys_it++ = it->impl();
+    *keys_it++ = 0;
+    d->lasterr = gpgme_op_import_keys( d->ctx, keys.get() );
+    shouldHaveResult = true;
+#endif
+    if ( ( gpgme_err_code( d->lasterr ) == GPG_ERR_NOT_IMPLEMENTED ||
+           gpgme_err_code( d->lasterr ) == GPG_ERR_NOT_SUPPORTED ) &&
+         protocol() == CMS )
+    {
+        // ok, try the workaround (export+import):
+        std::vector<const char*> fprs;
+        for ( std::vector<Key>::const_iterator it = kk.begin(), end = kk.end() ; it != end ; ++it ) {
+            if ( const char * fpr = it->primaryFingerprint() ) {
+                if ( *fpr )
+                    fprs.push_back( fpr );
+            } else if ( const char * keyid = it->keyID() ) {
+                if ( *keyid )
+                    fprs.push_back( keyid );
+            }
+        }
+        fprs.push_back( 0 );
+        Data data;
+        Data::Private * const dp = data.impl();
+        const gpgme_keylist_mode_t oldMode = gpgme_get_keylist_mode( d->ctx );
+        gpgme_set_keylist_mode( d->ctx, GPGME_KEYLIST_MODE_EXTERN );
+        d->lasterr = gpgme_op_export_ext( d->ctx, &fprs[0], 0, dp ? dp->data : 0 );
+        gpgme_set_keylist_mode( d->ctx, oldMode );
+        if ( !d->lasterr ) {
+            data.seek( 0, SEEK_SET );
+            d->lasterr = gpgme_op_import( d->ctx, dp ? dp->data : 0 );
+            shouldHaveResult = true;
+        }
+    }
+    if ( shouldHaveResult )
+        return ImportResult( d->ctx, Error(d->lasterr) );
+    else
+        return ImportResult( Error( d->lasterr ) );
+  }
+
   Error Context::startKeyImport( const Data & data ) {
     d->lastop = Private::Import;
     const Data::Private * const dp = data.impl();
     return Error( d->lasterr = gpgme_op_import_start( d->ctx, dp ? dp->data : 0 ) );
   }
 
+  Error Context::startKeyImport( const std::vector<Key> & kk ) {
+    d->lastop = Private::Import;
+#ifdef HAVE_GPGME_OP_IMPORT_KEYS
+    const boost::scoped_array<gpgme_key_t> keys( new gpgme_key_t[ kk.size() + 1 ] );
+    gpgme_key_t * keys_it = &keys[0];
+    for ( std::vector<Key>::const_iterator it = kk.begin(), end = kk.end() ; it != end ; ++it )
+      if ( it->impl() )
+        *keys_it++ = it->impl();
+    *keys_it++ = 0;
+    return Error( d->lasterr = gpgme_op_import_keys_start( d->ctx, keys.get() ) );
+#else
+    (void)kk;
+    return Error( d->lasterr = make_error( GPG_ERR_NOT_IMPLEMENTED ) );
+#endif
+  }
+
   ImportResult Context::importResult() const {
     if ( d->lastop & Private::Import )
       return ImportResult( d->ctx, Error(d->lasterr) );
@@ -424,6 +641,26 @@ namespace GpgME {
     return Error( d->lasterr = gpgme_op_delete_start( d->ctx, key.impl(), int( allowSecretKeyDeletion ) ) );
   }
 
+  Error Context::passwd( const Key & key ) {
+    d->lastop = Private::Passwd;
+#ifdef HAVE_GPGME_OP_PASSWD
+    return Error( d->lasterr = gpgme_op_passwd( d->ctx, key.impl(), 0U ) );
+#else
+    (void)key;
+    return Error( d->lasterr = make_error( GPG_ERR_NOT_IMPLEMENTED ) );
+#endif
+  }
+
+  Error Context::startPasswd( const Key & key ) {
+    d->lastop = Private::Passwd;
+#ifdef HAVE_GPGME_OP_PASSWD
+    return Error( d->lasterr = gpgme_op_passwd_start( d->ctx, key.impl(), 0U ) );
+#else
+    (void)key;
+    return Error( d->lasterr = make_error( GPG_ERR_NOT_IMPLEMENTED ) );
+#endif
+  }
+
   Error Context::edit( const Key & key, std::auto_ptr<EditInteractor> func, Data & data ) {
       d->lastop = Private::Edit;
       d->lastEditInteractor = func;
@@ -448,6 +685,10 @@ namespace GpgME {
       return d->lastEditInteractor.get();
   }
 
+  std::auto_ptr<EditInteractor> Context::takeLastEditInteractor() {
+      return d->lastEditInteractor;
+  }
+
   Error Context::cardEdit( const Key & key, std::auto_ptr<EditInteractor> func, Data & data ) {
       d->lastop = Private::CardEdit;
       d->lastCardEditInteractor = func;
@@ -472,6 +713,10 @@ namespace GpgME {
       return d->lastCardEditInteractor.get();
   }
 
+  std::auto_ptr<EditInteractor> Context::takeLastCardEditInteractor() {
+      return d->lastCardEditInteractor;
+  }
+
   Error Context::startTrustItemListing( const char * pattern, int maxLevel ) {
     d->lastop = Private::TrustList;
     return Error( d->lasterr = gpgme_op_trustlist_start( d->ctx, pattern, maxLevel ) );
@@ -487,6 +732,99 @@ namespace GpgME {
     return Error( d->lasterr = gpgme_op_trustlist_end( d->ctx ) );
   }
 
+#ifdef HAVE_GPGME_ASSUAN_ENGINE
+  static gpgme_error_t assuan_transaction_data_callback( void * opaque, const void * data, size_t datalen ) {
+    assert( opaque );
+    AssuanTransaction * t = static_cast<AssuanTransaction*>( opaque );
+    return t->data( static_cast<const char*>( data ), datalen ).encodedError();
+  }
+
+  static gpgme_error_t assuan_transaction_inquire_callback( void * opaque, const char * name, const char * args, gpgme_data_t * r_data ) {
+    assert( opaque );
+    Context::Private * p = static_cast<Context::Private*>( opaque );
+    AssuanTransaction * t = p->lastAssuanTransaction.get();
+    assert( t );
+    Error err;
+    if ( name )
+      p->lastAssuanInquireData = t->inquire( name, args, err );
+    else
+      p->lastAssuanInquireData = Data::null;
+    if ( !p->lastAssuanInquireData.isNull() )
+      *r_data = p->lastAssuanInquireData.impl()->data;
+    return err.encodedError();
+  }
+
+  static gpgme_error_t assuan_transaction_status_callback( void * opaque, const char * status, const char * args ) {
+    assert( opaque );
+    AssuanTransaction * t = static_cast<AssuanTransaction*>( opaque );
+    std::string a = args;
+    percent_unescape( a, true ); // ### why doesn't gpgme do this??
+    return t->status( status, a.c_str() ).encodedError();
+  }
+#endif
+
+  AssuanResult Context::assuanTransact( const char * command ) {
+    return assuanTransact( command, std::auto_ptr<AssuanTransaction>( new DefaultAssuanTransaction ) );
+  }
+
+  AssuanResult Context::assuanTransact( const char * command, std::auto_ptr<AssuanTransaction> transaction ) {
+    d->lastop = Private::AssuanTransact;
+    d->lastAssuanTransaction = transaction;
+    if ( !d->lastAssuanTransaction.get() )
+        return AssuanResult( Error( d->lasterr = make_error( GPG_ERR_INV_ARG ) ) );
+#ifdef HAVE_GPGME_ASSUAN_ENGINE
+    d->lasterr = gpgme_op_assuan_transact( d->ctx, command,
+                                           assuan_transaction_data_callback,
+                                           d->lastAssuanTransaction.get(),
+                                           assuan_transaction_inquire_callback,
+                                           d, // sic!
+                                           assuan_transaction_status_callback,
+                                           d->lastAssuanTransaction.get() );
+#else
+    (void)command;
+    d->lasterr = make_error( GPG_ERR_NOT_SUPPORTED );
+#endif
+    return AssuanResult( d->ctx, d->lasterr );
+  }
+
+  Error Context::startAssuanTransaction( const char * command ) {
+    return startAssuanTransaction( command, std::auto_ptr<AssuanTransaction>( new DefaultAssuanTransaction ) );
+  }
+
+  Error Context::startAssuanTransaction( const char * command, std::auto_ptr<AssuanTransaction> transaction ) {
+    d->lastop = Private::AssuanTransact;
+    d->lastAssuanTransaction = transaction;
+    if ( !d->lastAssuanTransaction.get() )
+        return Error( d->lasterr = make_error( GPG_ERR_INV_ARG ) );
+#ifdef HAVE_GPGME_ASSUAN_ENGINE
+    return Error( d->lasterr = gpgme_op_assuan_transact_start( d->ctx, command,
+                                                               assuan_transaction_data_callback,
+                                                               d->lastAssuanTransaction.get(),
+                                                               assuan_transaction_inquire_callback,
+                                                               d, // sic!
+                                                               assuan_transaction_status_callback,
+                                                               d->lastAssuanTransaction.get() ) );
+#else
+    (void)command;
+    return Error( d->lasterr = make_error( GPG_ERR_NOT_SUPPORTED ) );
+#endif
+  }
+
+  AssuanResult Context::assuanResult() const {
+    if ( d->lastop & Private::AssuanTransact )
+      return AssuanResult( d->ctx, d->lasterr );
+    else
+      return AssuanResult();
+  }
+
+  AssuanTransaction * Context::lastAssuanTransaction() const {
+    return d->lastAssuanTransaction.get();
+  }
+
+  std::auto_ptr<AssuanTransaction> Context::takeLastAssuanTransaction() {
+    return d->lastAssuanTransaction;
+  }
+
   DecryptionResult Context::decrypt( const Data & cipherText, Data & plainText ) {
     d->lastop = Private::Decrypt;
     const Data::Private * const cdp = cipherText.impl();
@@ -584,7 +922,7 @@ namespace GpgME {
     return Error( d->lasterr = gpgme_op_getauditlog_start( d->ctx, odp ? odp->data : 0, to_auditlog_flags( flags ) ) );
 #else
     (void)output; (void)flags;
-    return Error( d->lasterr = makeError( GPG_ERR_NOT_IMPLEMENTED ) );
+    return Error( d->lasterr = make_error( GPG_ERR_NOT_IMPLEMENTED ) );
 #endif
   }
 
@@ -595,7 +933,7 @@ namespace GpgME {
     return Error( d->lasterr = gpgme_op_getauditlog( d->ctx, odp ? odp->data : 0, to_auditlog_flags( flags ) ) );
 #else
     (void)output; (void)flags;
-    return Error( d->lasterr = makeError( GPG_ERR_NOT_IMPLEMENTED ) );
+    return Error( d->lasterr = make_error( GPG_ERR_NOT_IMPLEMENTED ) );
 #endif
   }
 
@@ -631,7 +969,7 @@ namespace GpgME {
     return Error( gpgme_sig_notation_add( d->ctx, name, value, add_to_gpgme_sig_notation_flags_t( 0, flags ) ) );
 #else
     (void)name; (void)value; (void)flags;
-    return Error( makeError( GPG_ERR_NOT_IMPLEMENTED ) );
+    return Error( make_error( GPG_ERR_NOT_IMPLEMENTED ) );
 #endif
   }
 
@@ -640,7 +978,7 @@ namespace GpgME {
     return Error( gpgme_sig_notation_add( d->ctx, 0, url, critical ? GPGME_SIG_NOTATION_CRITICAL : 0 ) );
 #else
     (void)url; (void)critical;
-    return Error( makeError( GPG_ERR_NOT_IMPLEMENTED ) );
+    return Error( make_error( GPG_ERR_NOT_IMPLEMENTED ) );
 #endif
   }
 
@@ -720,7 +1058,7 @@ namespace GpgME {
     d->lastop = Private::Encrypt;
 #ifndef HAVE_GPGME_ENCRYPT_NO_ENCRYPT_TO
     if ( flags & NoEncryptTo )
-        return EncryptionResult( Error( d->lasterr = gpg_error( GPG_ERR_NOT_IMPLEMENTED ) ) );
+        return EncryptionResult( Error( d->lasterr = make_error( GPG_ERR_NOT_IMPLEMENTED ) ) );
 #endif
     const Data::Private * const pdp = plainText.impl();
     Data::Private * const cdp = cipherText.impl();
@@ -748,7 +1086,7 @@ namespace GpgME {
     d->lastop = Private::Encrypt;
 #ifndef HAVE_GPGME_ENCRYPT_NO_ENCRYPT_TO
     if ( flags & NoEncryptTo )
-        return Error( d->lasterr = gpg_error( GPG_ERR_NOT_IMPLEMENTED ) );
+        return Error( d->lasterr = make_error( GPG_ERR_NOT_IMPLEMENTED ) );
 #endif
     const Data::Private * const pdp = plainText.impl();
     Data::Private * const cdp = cipherText.impl();
@@ -804,6 +1142,42 @@ namespace GpgME {
     return Error( d->lasterr );
   }
 
+  Error Context::createVFS(const char* containerFile, const std::vector< Key >& recipients) {
+    d->lastop = Private::CreateVFS;
+#ifdef HAVE_GPGME_G13_VFS
+    gpgme_key_t * const keys = new gpgme_key_t[ recipients.size() + 1 ];
+    gpgme_key_t * keys_it = keys;
+    for ( std::vector<Key>::const_iterator it = recipients.begin() ; it != recipients.end() ; ++it )
+      if ( it->impl() )
+        *keys_it++ = it->impl();
+    *keys_it++ = 0;
+
+    gpgme_error_t op_err;
+    d->lasterr = gpgme_op_vfs_create( d->ctx, keys, containerFile, 0, &op_err );
+    delete[] keys;
+    Error error( d->lasterr );
+    if ( error )
+      return error;
+    return Error( d->lasterr = op_err );
+#else
+    Q_UNUSED( containerFile );
+    Q_UNUSED( recipients );
+    return Error( d->lasterr = make_error( GPG_ERR_NOT_SUPPORTED ) );
+#endif
+  }
+
+  VfsMountResult Context::mountVFS(const char* containerFile, const char* mountDir) {
+    d->lastop = Private::MountVFS;
+#ifdef HAVE_GPGME_G13_VFS
+    gpgme_error_t op_err;
+    d->lasterr = gpgme_op_vfs_mount( d->ctx, containerFile, mountDir, 0, &op_err );
+    return VfsMountResult( d->ctx, Error( d->lasterr ), Error( op_err ) );
+#else
+    Q_UNUSED( containerFile );
+    Q_UNUSED( mountDir );
+    return VfsMountResult( d->ctx, Error( d->lasterr = make_error( GPG_ERR_NOT_SUPPORTED ) ), Error() );
+#endif
+  }
 
   Error Context::cancelPendingOperation() {
 #ifdef HAVE_GPGME_CANCEL_ASYNC
@@ -921,7 +1295,7 @@ namespace GpgME {
 
   std::ostream & operator<<( std::ostream & os, Context::EncryptionFlags flags ) {
       os << "GpgME::Context::EncryptionFlags(";
-#define CHECK( x ) if ( !(flags & (Context::x)) ) {} else do { os << #x " "; } while (0) 
+#define CHECK( x ) if ( !(flags & (Context::x)) ) {} else do { os << #x " "; } while (0)
      CHECK( AlwaysTrust );
 #undef CHECK
       return os << ')';
@@ -981,6 +1355,12 @@ static gpgme_protocol_t engine2protocol( const GpgME::Engine engine ) {
         return GPGME_PROTOCOL_ASSUAN;
 #else
         break;
+#endif
+    case GpgME::G13Engine:
+#ifdef HAVE_GPGME_G13_VFS
+        return GPGME_PROTOCOL_G13;
+#else
+        break;
 #endif
     case GpgME::UnknownEngine:
         ;
@@ -1087,8 +1467,26 @@ static const unsigned long supported_features = 0
 #ifdef HAVE_GPGME_KEYLIST_MODE_EPHEMERAL
     | GpgME::EphemeralKeylistModeFeature
 #endif
+#ifdef HAVE_GPGME_OP_IMPORT_KEYS
+    | GpgME::ImportFromKeyserverFeature
+#endif
+#ifdef HAVE_GPGME_G13_VFS
+    | GpgME::G13VFSFeature
+#endif
+#ifdef HAVE_GPGME_OP_PASSWD
+    | GpgME::PasswdFeature
+#endif
+    ;
+
+static const unsigned long supported_features2 = 0
     ;
 
 bool GpgME::hasFeature( unsigned long features ) {
     return features == ( features & supported_features );
 }
+
+bool GpgME::hasFeature( unsigned long features, unsigned long features2 ) {
+    return features  == ( features  & supported_features  )
+        && features2 == ( features2 & supported_features2 )
+        ;
+}
