@@ -191,6 +191,10 @@ class KMIME_EXPORT Base
 
     /**
       Returns the decoded content of the header without the header-type.
+
+      @note The return value of this method should only be used when showing an address
+            to the user. It is not guaranteed that fromUnicodeString( asUnicodeString(), ... )
+            will return the original string.
     */
     virtual QString asUnicodeString() const = 0;
 
@@ -714,6 +718,9 @@ class KMIME_EXPORT Parametrized : public Structured
     virtual bool isEmpty() const;
     virtual void clear();
 
+    //FIXME: Shouldn't the parameter keys be QByteArray and not QStrings? Only the values can be
+    //       non-ascii!
+
     /**
       Returns the value of the specified parameter.
       @param key The parameter name.
@@ -1485,6 +1492,10 @@ class KMIME_EXPORT UserAgent : public Generics::Unstructured
   kmime_mk_trivial_ctor_with_name( UserAgent )
 };
 
+/** Creates a header based on @param type. If @param type is a known header type,
+ * the right object type will be created, otherwise a null pointer is returned. */
+KMIME_EXPORT Base *createHeader( const QByteArray& type );
+
 }  //namespace Headers
 
 }  //namespace KMime
