@@ -28,6 +28,8 @@
 #include "kmime_headers.h"
 #include "boolflags.h"
 
+#include <QtCore/QMetaType>
+
 namespace boost {
   template <typename T> class shared_ptr;
 }
@@ -230,4 +232,7 @@ class KMIME_EXPORT Message : public Content
 
 } // namespace KMime
 
+#define KMIME_MESSAGE_METATYPE_DEFINED 1
+Q_DECLARE_METATYPE(KMime::Message*)
+
 #endif // __KMIME_MESSAGE_H__
