@@ -18,23 +18,24 @@
 */
 
 #include "transportmanager.h"
-#include "addtransportdialog.h"
 #include "resourcesendjob_p.h"
 #include "mailtransport_defs.h"
-#include "sendmailconfigwidget.h"
 #include "sendmailjob.h"
-#include "smtpconfigwidget.h"
 #include "smtpjob.h"
 #include "transport.h"
-#include "transportconfigwidget.h"
 #include "transportjob.h"
 #include "transporttype.h"
 #include "transporttype_p.h"
+#include "addtransportdialog.h"
 #include "transportconfigdialog.h"
+#include "transportconfigwidget.h"
+#include "sendmailconfigwidget.h"
+#include "smtpconfigwidget.h"
 
 #include <QApplication>
 #include <QtDBus/QDBusConnection>
 #include <QtDBus/QDBusConnectionInterface>
+#include <QtDBus/QDBusServiceWatcher>
 #include <QPointer>
 #include <QRegExp>
 #include <QStringList>
@@ -42,7 +43,7 @@
 #include <KConfig>
 #include <KConfigGroup>
 #include <KDebug>
-#include <kemailsettings.h>
+#include <KEMailSettings>
 #include <KLocale>
 #include <KMessageBox>
 #include <KRandom>
@@ -97,9 +98,7 @@ class TransportManagerPrivate
     // Slots
     void slotTransportsChanged();
     void slotWalletOpened( bool success );
-    void dbusServiceOwnerChanged( const QString &service,
-                                  const QString &oldOwner,
-                                  const QString &newOwner );
+    void dbusServiceUnregistered();
     void agentTypeAdded( const Akonadi::AgentType &atype );
     void agentTypeRemoved( const Akonadi::AgentType &atype );
     void jobResult( KJob *job );
@@ -123,6 +122,7 @@ TransportManager::TransportManager()
   : QObject(), d( new TransportManagerPrivate( this ) )
 {
   KGlobal::locale()->insertCatalog( QLatin1String( "libmailtransport" ) );
+  KGlobal::locale()->insertCatalog( QLatin1String( "libakonadi-kmime" ) );
   qAddPostRoutine( destroyStaticTransportManager );
   d->myOwnChange = false;
   d->appliedChange = false;
@@ -133,18 +133,18 @@ TransportManager::TransportManager()
   d->config = new KConfig( QLatin1String( "mailtransports" ) );
 
   QDBusConnection::sessionBus().registerObject( DBUS_OBJECT_PATH, this,
-      QDBusConnection::ExportScriptableSlots |
-              QDBusConnection::ExportScriptableSignals );
+                                                QDBusConnection::ExportScriptableSlots |
+                                                QDBusConnection::ExportScriptableSignals );
+
+  QDBusServiceWatcher *watcher = new QDBusServiceWatcher( DBUS_SERVICE_NAME, QDBusConnection::sessionBus(),
+                                                          QDBusServiceWatcher::WatchForUnregistration, this );
+  connect( watcher, SIGNAL( serviceUnregistered( const QString& ) ), SLOT( dbusServiceUnregistered() ) );
 
   QDBusConnection::sessionBus().connect( QString(), QString(),
                               DBUS_INTERFACE_NAME, DBUS_CHANGE_SIGNAL,
                               this, SLOT(slotTransportsChanged()) );
 
-  d->isMainInstance =
-          QDBusConnection::sessionBus().registerService( DBUS_SERVICE_NAME );
-  connect( QDBusConnection::sessionBus().interface(),
-           SIGNAL(serviceOwnerChanged(QString,QString,QString)),
-           SLOT(dbusServiceOwnerChanged(QString,QString,QString)) );
+  d->isMainInstance = QDBusConnection::sessionBus().registerService( DBUS_SERVICE_NAME );
 
   d->fillTypes();
 }
@@ -753,14 +753,9 @@ void TransportManagerPrivate::migrateToWallet()
   }
 }
 
-void TransportManagerPrivate::dbusServiceOwnerChanged( const QString &service,
-                                                         const QString &oldOwner,
-                                                         const QString &newOwner )
+void TransportManagerPrivate::dbusServiceUnregistered()
 {
-  Q_UNUSED( oldOwner );
-  if ( service == DBUS_SERVICE_NAME && newOwner.isEmpty() ) {
-    QDBusConnection::sessionBus().registerService( DBUS_SERVICE_NAME );
-  }
+  QDBusConnection::sessionBus().registerService( DBUS_SERVICE_NAME );
 }
 
 void TransportManagerPrivate::agentTypeAdded( const Akonadi::AgentType &atype )
